<?php

namespace Drupal\rabbit_hole\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\rabbit_hole\BehaviorSettingsInterface;

/**
 * Defines the Behavior settings entity.
 *
 * @ConfigEntityType(
 *   id = "behavior_settings",
 *   label = @Translation("Rabbit hole settings"),
 *   handlers = {},
 *   config_prefix = "behavior_settings",
 *   admin_permission = "administer site configuration",
 *   entity_keys = {
 *     "id" = "id",
 *     "uuid" = "uuid",
 *     "action" = "action"
 *   },
 *   config_export = {
 *     "id",
 *     "entity_type_id",
 *     "entity_id",
 *     "uuid",
 *     "action",
 *     "no_bypass",
 *     "bypass_message",
 *     "configuration"
 *   },
 *   links = {}
 * )
 */
class BehaviorSettings extends ConfigEntityBase implements BehaviorSettingsInterface {

  /**
   * The Behavior settings ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The configured action (e.g. display_page).
   *
   * @var string
   */
  protected $action;

  /**
   * The entity type id, eg. 'node_type'.
   *
   * @var string
   */
  protected $entity_type_id;

  /**
   * The entity id, eg. 'article'.
   *
   * @var string
   */
  protected $entity_id;

  /**
   * The bypass action.
   *
   * @var bool
   */
  protected $no_bypass;

  /**
   * The action-specific configuration.
   *
   * @var array
   */
  protected array $configuration = [];

  /**
   * {@inheritdoc}
   */
  public function setAction($action) {
    $this->action = $action;
  }

  /**
   * {@inheritdoc}
   */
  public function getAction() {
    return $this->action;
  }

  /**
   * {@inheritdoc}
   */
  public function setNoBypass(bool $no_bypass) {
    $this->no_bypass = $no_bypass;
  }

  /**
   * {@inheritdoc}
   */
  public function getNoBypass(): bool {
    return $this->no_bypass;
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    parent::calculateDependencies();

    if ($this->entity_type_id && $this->entity_id) {
      // Create dependency on the bundle.
      $bundle = \Drupal::entityTypeManager()->getDefinition($this->entity_type_id);
      $entity_type = \Drupal::entityTypeManager()->getDefinition($bundle->getBundleOf());
      $bundle_config_dependency = $entity_type->getBundleConfigDependency($this->entity_id);
      $this->addDependency($bundle_config_dependency['type'], $bundle_config_dependency['name']);
    }

    return $this;
  }

}
