<?php

namespace Drupal\Tests\rabbit_hole\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\node\Entity\Node;
use Drupal\rabbit_hole\Plugin\RabbitHoleBehaviorPlugin\PageNotFound;
use Drupal\Tests\node\Traits\ContentTypeCreationTrait;
use Drupal\Tests\user\Traits\UserCreationTrait;

/**
 * Test cases for BehaviorInvoker.
 *
 * @coversDefaultClass \Drupal\rabbit_hole\BehaviorInvoker
 * @group rabbit_hole
 */
class BehaviorInvokerTest extends KernelTestBase {

  use ContentTypeCreationTrait;
  use UserCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'filter',
    'text',
    'field',
    'user',
    'node',
    'rabbit_hole',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('node');
    $this->installEntitySchema('user');
    $this->installConfig(['filter', 'node', 'system', 'rabbit_hole']);

    \Drupal::service('rabbit_hole.behavior_settings_manager')->enableEntityType('node');

    $this->createContentType(['type' => 'article']);
    $this->createContentType(['type' => 'page']);

    \Drupal::service('rabbit_hole.behavior_settings_manager')->saveBehaviorSettings([
      'action' => 'page_not_found',
      'no_bypass' => FALSE,
    ], 'node_type', 'article');

    \Drupal::service('rabbit_hole.behavior_settings_manager')->saveBehaviorSettings([
      'action' => 'page_not_found',
      'no_bypass' => TRUE,
    ], 'node_type', 'page');
  }

  /**
   * @covers ::getBehaviorPlugin()
   */
  public function testGetBehaviorPlugin() {
    $node1 = Node::create(['title' => '#freeAzov', 'type' => 'article']);
    $node1->save();
    $node2 = Node::create(['title' => 'Support Ukraine', 'type' => 'page']);
    $node2->save();

    $behavior_invoker = \Drupal::service('rabbit_hole.behavior_invoker');

    $this->setUpCurrentUser([], [], TRUE);
    // "No bypass" for articles is disabled, so admin should see the page.
    // In other words, the plugin should be not available.
    $this->assertNull($behavior_invoker->getBehaviorPlugin($node1));
    // For pages, "no bypass" is enabled, so action plugin is expected.
    $this->assertInstanceOf(PageNotFound::class, $behavior_invoker->getBehaviorPlugin($node2));

    // Verify that regular user cannot access article page.
    $this->setUpCurrentUser();
    $this->assertInstanceOf(PageNotFound::class, $behavior_invoker->getBehaviorPlugin($node1));
  }

}
