<?php

namespace Drupal\Tests\rabbit_hole\Kernel;

use Drupal\Core\DrupalKernel;
use Drupal\KernelTests\KernelTestBase;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\user\Entity\User;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Test cases for BehaviorSettingsManager.
 *
 * @coversDefaultClass \Drupal\rabbit_hole\BehaviorSettingsManager
 * @group rabbit_hole
 */
class BehaviorSettingsManagerTest extends KernelTestBase {

  use UserCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['rabbit_hole', 'node', 'user'];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->setUpCurrentUser();
    $this->installEntitySchema('user');
    $this->installEntitySchema('node');
    $this->installConfig('rabbit_hole');
    \Drupal::service('rabbit_hole.behavior_settings_manager')->enableEntityType('node');
  }

  /**
   * @covers ::getEntity()
   */
  public function testGetEntity() {
    $behavior_invoker = \Drupal::service('rabbit_hole.behavior_invoker');
    $class_loader = require $this->root . '/autoload.php';
    $kernel = new DrupalKernel('testing', $class_loader, FALSE);

    // Supported, not enabled.
    User::create(['name' => 'borisjohnsonuk'])->save();
    $event = new RequestEvent($kernel, $this->createRequest('/user/1'), HttpKernelInterface::MASTER_REQUEST);
    $this->assertEquals(FALSE, $behavior_invoker->getEntity($event));

    // Enabled.
    Node::create(['title' => 'God Save the King', 'type' => 'ty'])->save();
    $event = new RequestEvent($kernel, $this->createRequest('/node/1'), HttpKernelInterface::MASTER_REQUEST);
    $this->assertInstanceOf(NodeInterface::class, $behavior_invoker->getEntity($event));
  }

  /**
   * Creates a request object for given path.
   *
   * @param string $uri
   *   A URI or path.
   *
   * @return \Symfony\Component\HttpFoundation\Request
   *   Created request object.
   */
  protected function createRequest(string $uri): Request {
    $request = Request::create($uri);
    $parameters = \Drupal::service('router.no_access_checks')->matchRequest($request);
    $request->attributes->add($parameters);
    unset($parameters['_route'], $parameters['_controller']);
    $request->attributes->set('_route_params', $parameters);
    return $request;
  }

}
